function J = numerical_Jacobian(funfile,X0,h,varargin)
% NUMERICAL_JACOBIAN(FUN,X,h,varargin)
%   Computes the Numerical Jacobian of the function FUN at the point X0 with
%   a finite difference step-size of h. Forward difference is used for the
%   derivative estimates.
%   Assumptions: X0 is a column vector and h is a scalar.
%   FUN is a function of the single vector variable X0 and returns a column
%   vector. varargin are extra parameter inputs to FUN.


tempsize = size(X0); 
if tempsize(2)~=1
    error('Input X0 is not a column vector.');
end
f0 = feval(funfile,X0,varargin{:});
tempsize = size(f0); 
if tempsize(2)~=1
    error('Function output f0 is not a column vector.');
end

JT = zeros(length(X0),length(f0)); % compute J transpose first
for i=1:length(X0)
    temp = X0(i)+h;
    Xhi = X0; Xhi(i) = temp;
    f = feval(funfile,Xhi,varargin{:});
    delf_delxi = (f-f0)/h;
    JT(i,:) = delf_delxi;
end
J = JT';
end