
clc; clear; close all;

%% equation of motion is
% xdotdot + + epsilon*c*xdot + k*x + epsilon*alpha*x^3  = A*cos(omega*t)

% hardening spring
param.c = 0.05; param.k = 1; param.epsilon = 0.5;
param.c = 0.1; param.k = 1; param.alpha = 1; param.epsilon = 0.1;
param.A = 1;

% softening spring
% param.c = 0.2; param.k = 1; 
% param.alpha = -4;  % cubic nonlinearity, softening
% param.epsilon = 0.1; % small parameter that keeps damping and nonlinearity small
% param.A = 0.02; % forcing amplitude


anotherSweep = input('Do you want to do another frequency sweep? Type y or n: ','s');
%%

%% frequency and response amplitude range
numFrequencies = 40;
omegaMin = 0.1;
omegaMax = 4;
deltaOmega = 0.05;

responseMin = 0;
responseMax = 3;

figure(1);
xlim([omegaMin omegaMax]);
ylim([responseMin responseMax]);
xlabel('forcing frequency');
ylabel('response amplitude');
[omegaClicked,responseClicked] = ginput(1);
close;
Y0 = [responseClicked; 0];


omegaStore = []; amplitudeStore = [];

%% good initial seed for unstable branch, for the above parameters
Y0CheatHardening = [-2.524605269375743; 0.128843839317501];
omegaCheatHardening = 1.368750000000000;

%% good initial seed for unstable branch, for the above parameters
Y0CheatSoftening = [0.562517837006748; 0.708921910532073];
omegaCheatSoftening = 0.8250000000;

%%
while strcmp(anotherSweep,'y')
    
    
    while (omegaClicked<omegaMin)||(omegaClicked>omegaMax)
        warning('Please click again WITHIN the box!');
        [omegaClicked,responseClicked] = ginput(1);
    end
    
    
    forwardOrBackward = input('Forward or Backward sweep? Type f or b: ','s');
    
    %%
    if strcmp(forwardOrBackward,'f')
        numOmega = round((omegaMax-omegaClicked)/deltaOmega);
        omegalist = linspace(omegaClicked,omegaMax,numOmega);
    else
        numOmega = round((omegaClicked-omegaMin)/deltaOmega);
        omegalist = linspace(omegaClicked,omegaMin,numOmega);
    end
    
    amplitudelist = zeros(size(omegalist));
    
    
    
    
    
    YresultStore = [];
    for i = 1:numOmega
        param.omega = omegalist(i); % forcing frequency for the Duffing oscillator
        
        % Using the root find to solve for a fixed point of the Poincare map and
        % therefor an initial condition for the limit cycle, assuming fsolve
        % converges
        display('Using fsolve to find the fixed point of the Poincare map and therefore an initial condition on the limit cycle');
        options = optimset('display','iter','MaxIter',30);
        Yresult = fsolve(@differenceYiYiplus1,Y0,options,param);
        
        YresultStore = [YresultStore Yresult];
        
        howgoodistheresult = differenceYiYiplus1(Yresult,param);
        
        if norm(howgoodistheresult)<1e-5 % successfully found a periodic motion
            
            %% find amplitude by simulating again
            amplitudelist(i) = findamplitude_ForcedDuffing(Yresult,param);
            
            %% We now compute the Jacobian of the Poincare map so we can examine the
            % stability of the limit cycle
            h = 1e-5; % because the ODEs give about 1e-9 accuracy. optimal finite difference step size
            J = numerical_Jacobian(@OneCycleMap_ForcedDuffing,Yresult,h,param)
            
            % doing stability analysis with Jacobian requires VERY high
            % accuracy of periodic motion, not obtainable by
            % simulation, but only through fsolve and Poincare map
            
            eigVals = eig(J)
            % checking if all the eigenvalues are less than one in absolute value
            if all(abs(eigVals)<1)
                %         display('All eigenvalues are less than 1 in absolute value, hence stable limit cycle');
                stableornot = 1;
                display('This is a stable limit cycle, because all eigenvalues have absolute values less than 1');
                figure(1); hold on;
                plot(omegalist(i),amplitudelist(i),'bo');
                xlim([omegaMin omegaMax]);
            else
                %         display('At least one eigenvalue is greater than or equal to 1 in absolute value');
                stableornot = 0;
                display('This is an unstable limit cycle, because NOT all eigenvalues have absolute values less than 1');
                figure(1); hold on;
                plot(omegalist(i),amplitudelist(i),'ro');
                xlim([omegaMin omegaMax]);
            end
            
        else
            amplitudelist(i) = NaN; % setting NaN amplitude when no periodic motion was found
        end
        
        % stop for loop if we have failed to converge to a periodic motion
        if isnan(amplitudelist(i))
            warning('stopped because the last one was not a periodic orbit');
            break
        elseif i ==1
            Y0 = Yresult; % use the current result as initial condition for next root find
            omegaStore = [omegaStore; omegalist(i)];
            amplitudeStore = [amplitudeStore; amplitudelist(i)];
        else
            Y0 = YresultStore(:,i)+(YresultStore(:,i)-YresultStore(:,i))/(omegalist(i)-omegalist(i-1));
            omegaStore = [omegaStore; omegalist(i)];
            amplitudeStore = [amplitudeStore; amplitudelist(i)];
            
        end
        %
        
    end
    
    anotherSweep = input('Do you want to do another frequency sweep? Type y or n: ','s');
    
    
    if strcmp(anotherSweep,'y')
        clickOrCheat = input('Do you want to click or cheat (use stored Initial Seed)? say y or n: ','s');
        
        if strcmp(clickOrCheat,'y')
            figure(1);
            xlim([omegaMin omegaMax]);
            ylim([responseMin max(amplitudeStore)]);
            xlabel('forcing frequency');
            ylabel('response amplitude');
            [omegaClicked,responseClicked] = ginput(1);
            Y0 = [responseClicked; 0];
            
        else
            if param.alpha>0 % hardening unstable branch
                Y0 = Y0CheatHardening;
                omegaClicked = omegaCheatHardening;
                % use a previously stored unstable periodic motion obtained
                % from a random initial seed
            else
                Y0 = Y0CheatSoftening; % softening unstable branch
                omegaClicked = omegaCheatSoftening;
            end
            
        end
        
    end
        
    end
    
    
    
