% this program finds a fixed point for the Poincare Map, thereby finding 
% an initial condition that is on the limit cycle and the Poincare Section

% as we know, this method is better than just finding the steady response amplitude
% by simulation, because this method can find both stable and unstable
% limit cycles, depending on the initial guess

% We also find the stability of the limit cycle here.


clc; clear all; close all;

omega = 2; % forcing frequency for the Duffing oscillator

Y0 = [0.3; 0]; 
% initial guess, but this may not be on the right side of the Poincare section
% so we integrate once until we hit the Poincare section and take that as
% the initial seed
% Y0 = PoincareMap_ForcedDuffing(Y0,omega);

% Using the root find to solve for a fixed point of the Poincare map and
% therefor an initial condition for the limit cycle, assuming fsolve
% converges
display('Using fsolve to find the fixed point of the Poincare map and therefore an initial condition on the limit cycle');
Yresult = fsolve(@differenceYiYiplus1,Y0,[],omega);

howgoodistheresult = differenceYiYiplus1(Yresult,omega)

%% We now compute the Jacobian of the Poincare map so we can examine the
% stability of the limit cycle
J = numerical_Jacobian(@PoincareMap_ForcedDuffing,Yresult,10^(-5),omega)
eigVals = eig(J)
% checking if all the eigenvalues are less than one in absolute value
if all(abs(eigVals)<1)
%         display('All eigenvalues are less than 1 in absolute value, hence stable limit cycle');
    stableornot = 1;
    display('This is a stable limit cycle, because all eigenvalues have absolute values less than 1');
else
%         display('At least one eigenvalue is greater than or equal to 1 in absolute value');
    stableornot = 0;
    display('This is an unstable limit cycle, because NOT all eigenvalues have absolute values less than 1');
end
    